// Journey of Fate re-factored
// by Dennis Carr AKA ihavefivehat
// 2013

// Import sound library:
import ddf.minim.spi.*;
import ddf.minim.signals.*;
import ddf.minim.*;
import ddf.minim.analysis.*;
import ddf.minim.ugens.*;
import ddf.minim.effects.*;

// Objects for audio playback
Minim minim;
Sounder music;

// The game's font:
PFont font;

// Booleans for interaction
boolean click, did_once, state_change;

float chance_to_trip, trip;

// The current state of the game
int state;

// The current scene
int scene_number;

// The number of scenes in total
int number_of_scenes;

// I don't know what this is:
int text_state;

String[] fairyDialogue;

// For remembering the order that the actors should appear in.
ArrayList<Actor> sceneActors;

// The various actors
Player player;
Wizard wizard;
Monster monster;
Fairy fairy;
Villagers villagers;


Alarm mouseTimer;


void setup() {

  size(800, 600);

  // Initialize the game's font:
  font = loadFont("TitleScreen.vlw");

  // Initialize minim and the Sounder object which plays the music
  minim = new Minim(this);
  music = new Sounder();

  mouseTimer = new Alarm(0);

  // Initialize non-actor objects
  player = new Player();
  
  chance_to_trip = 0.0;
  trip = 1;

  // Assign each actor a certain race
  StringList possibleRaces = new StringList();
  possibleRaces.append("sandwich");
  possibleRaces.append("dodecahedron");
  possibleRaces.append("crab");
  possibleRaces.append("poop");
  possibleRaces.shuffle();
  String wizardRace = possibleRaces.get(0);
  String monsterRace = possibleRaces.get(1);
  String fairyRace = possibleRaces.get(2);
  String villagersRace = possibleRaces.get(3);

  // Assign each actor to a certain location:
  StringList possibleLocations = new StringList();
  possibleLocations.append("forest");
  possibleLocations.append("desert");
  possibleLocations.append("mountains");
  possibleLocations.append("caves");
  possibleLocations.shuffle();
  String wizardLocation = possibleLocations.get(0);
  String monsterLocation = possibleLocations.get(1);
  String fairyLocation = possibleLocations.get(2);
  String villagersLocation = possibleLocations.get(3);

  // Initialize actors:
  wizard = new Wizard(wizardRace, wizardLocation);
  monster = new Monster(monsterRace, monsterLocation);
  fairy = new Fairy(fairyRace, fairyLocation);
  villagers = new Villagers(villagersRace, villagersLocation);

  // Decide which actors will appear in which scenes:
  number_of_scenes = 20;
  sceneActors = new ArrayList<Actor>();

  for (int i = 0; i < number_of_scenes; i++) {    

    // Choose a random actor for this scene
    int actor_choice = int( random(4) );
    // Put that actor into sceneActors
    if      (actor_choice == 0) sceneActors.add(fairy);
    else if (actor_choice == 1) sceneActors.add(monster);
    else if (actor_choice == 2) sceneActors.add(villagers);
    else if (actor_choice == 3) sceneActors.add(wizard);
    
    // If the same actor is appearing in the last three scenes, do it over.
    if ( i > 1 && sceneActors.get(i-2) == sceneActors.get(i-1) && sceneActors.get(i-1) == sceneActors.get(i) ) {
      sceneActors.remove(i);
      i -= 1;
    } else {
      Actor currentActor = sceneActors.get(i);
      println("Scene " +i+ ": " +currentActor.race_name+ " " +currentActor.actor_name+
        " of the " +currentActor.biome_name);
    }
  }

  // Set various things:
  click = false;
  did_once = false;

  state = 0;
}


void draw() {
  // Play the music
  music.play();

  // Run the mouse timer
  mouseTimer.run();

  if (state == 0) {
    displayTitle();
    if (click) state++;
  } else if (state == 1) {
    displayInstructionalText();
    if (click) state++;
  }


  // Actor scenes
  else if (state > 1 && state < 99) {
    scene_number = state-2;
    Actor currentActor;

    // Get the actor for this scene
    if (scene_number < number_of_scenes-1 && chance_to_trip < trip) {
      currentActor = sceneActors.get(scene_number);
//      monster.dead = true;
//      currentActor = villagers;
      currentActor.display();
      currentActor.action();
    }
    else if (scene_number >= number_of_scenes-1) {
      currentActor = sceneActors.get(scene_number-1);
      currentActor.display();
      textBox("You turn around to leave and suddenly trip. Your head smashes into a rock and splits open gruesomely. Oops! Guess that's just how things go.");
      player.dead = true;
    }
    else {
      currentActor = sceneActors.get(scene_number-1);
      currentActor.display();
      textBox("You turn around to leave and suddenly trip. Your head smashes into a rock and splits open gruesomely. Oops! Guess that's just how things go.");
      player.dead = true;
    }

    if (click) {      
      // If the player is dead, show the game over screen
      if (player.dead) {
        state = 99;
      }
      
      // If the player won the game, show the win screen
      if (player.won_game) {
        state = 100;
      }
            
      // 1: Check if any variables need to be changed. If so, change them.
      
      
      // If the player should gain health, do it.
      if (player.gain_health) {
        player.health += 1;
        player.gain_health = false;
      }
      
      // If the player should lose health, do it.
      if (player.apply_damage) {
        player.health -= currentActor.attack_power;
        player.apply_damage = false;
      }
      
      // If the current actor should loose health, do it
      if (currentActor.apply_damage) {
        currentActor.health -= player.attack_power;
        currentActor.apply_damage = false;
      }
      
      if (player.gain_special_hug) {
        player.special_hug_obtained = true;
        player.gain_special_hug = false;
      }

      // 2: Handle dialgoue
      
      currentActor.line_index++;
      
      // If the current dialogue is finished, load the next dialouge and reset the line
      // index.
      if (currentActor.dialogue_finished) {
        currentActor.currentDialogue = currentActor.nextDialogue;
        currentActor.line_index = 0;
        currentActor.dialogue_finished = false;
      }

      // If the scene is finished, reset the current actor's phrase_index and then proceed to
      // the next state.
      if (currentActor.scene_finished) {
        println("Scene " +scene_number+ " finished");
        trip = random(1);
        if (scene_number > 10) chance_to_trip += 0.05;
        println(chance_to_trip);
        currentActor.reset();
        currentActor.scene_finished = false;
        state++;
        state = constrain(state, 0, number_of_scenes+1);
      }
    }
  }
  else if (state == 99) {
    gameOver();
  }
  else if (state == 100) {
    win();
  }

  click = false;
}


void mouseClicked() {
  if (mouseTimer.finished) {
    click = true;
    mouseTimer = new Alarm(10);
  }
}
